"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _interopRequireWildcard2 = _interopRequireDefault(require("@babel/runtime/helpers/interopRequireWildcard"));

var _plugin = _interopRequireDefault(require("@start/plugin"));

var _default = outDirRelative => (0, _plugin.default)('typescriptGenerate', ({
  logPath
}) => async ({
  files
}) => {
  const path = await Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require('path')));
  const ts = await Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require('typescript')));
  const filePaths = files.map(file => file.path);

  for (const filePath of filePaths) {
    logPath(filePath);
    const fileDir = path.dirname(filePath);
    const configPath = ts.findConfigFile(fileDir, ts.sys.fileExists);

    if (typeof configPath === 'undefined') {
      throw new Error(`Unable to find \`tsconfig.json\` for ${fileDir}`);
    }

    const configFile = ts.readConfigFile(configPath, ts.sys.readFile);
    const parsedConfig = ts.parseJsonConfigFileContent(configFile.config, ts.sys, fileDir);
    const options = { ...parsedConfig.options,
      noEmit: false,
      emitDeclarationOnly: true,
      declarationDir: path.resolve(outDirRelative),
      declaration: true
    };
    const program = ts.createProgram(filePaths, options);
    const emitResult = program.emit();
    const allDiagnostics = ts.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);

    for (const diagnostic of allDiagnostics) {
      if (typeof diagnostic.file === 'undefined') {
        console.log(`${ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n')}`);
        continue;
      }

      const {
        line,
        character
      } = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
      const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
      console.log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
    }
  }

  return {
    files
  };
});

exports.default = _default;